# coding: utf-8
u"""
.. Note::
    Сниппет выдернут из django_extensions и переделан под свои нужды
    Переделана сериализация, вместо simplejson используется json.
    Добавлена сериализация модели в словарь.

JSONField автоматически сериализует большинство Python элементов в JSON.
Создается текстовое поле cо значением по умолчанию "{}". Для получения
дополнительной информации см. test_json.py.

 class LOL(models.Model):
     extra = json.JSONField()

"""
import json
from decimal import Decimal
from django.db import models


def serialize_model(instance):
    u""" Сериализует экземпляр модели в JSON. """
    fields_dict = instance.__dict__.copy()
    for field_name in fields_dict.keys():
        if field_name.startswith('_'):
            del fields_dict[field_name]
    return json.dumps(fields_dict, cls=JSONEncoder)


def dumps(value):
    assert isinstance(value, (dict, models.Model))
    if isinstance(value, dict):
        dump = json.dumps(value, cls=JSONEncoder)
    else:
        dump = serialize_model(value)
    return dump


def loads(txt):
    value = json.loads(txt)
    assert isinstance(value, dict)
    return value


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Decimal):
            return float(obj)
        elif hasattr(obj, 'isoformat'):
            return obj.isoformat()

        try:
            ret = json.JSONEncoder.default(self, obj)
        except TypeError:
            ret = unicode(obj)
        return ret


class JSONDict(dict):
    u"""
    Hack so repr() called by dumpdata will output JSON instead of
    Python formatted data.  This way fixtures will work!
    """
    def __repr__(self):
        return dumps(self)


class JSONField(models.TextField):
    u""" JSONField is a generic textfield stores the JSON.

    JSONField is a generic textfield that neatly serializes/
    unserializes JSON objects seamlessly. Main thingy must be
    a dict object.

    """
    #: Used so to_python() is called
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = '{}'
        models.TextField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        u""" Конвертирование из строки в JSON после загрузки из DB. """
        if not value:
            return {}
        elif isinstance(value, basestring):
            res = loads(value)
            assert isinstance(res, dict)
            return JSONDict(**res)
        else:
            return value

    def get_db_prep_save(self, value, connection=None):
        u""" Конвертирование JSON объекта в строку перед сохранением. """
        if not value:
            return super(JSONField, self).get_db_prep_save("", connection=connection)
        else:
            return super(JSONField, self).get_db_prep_save(dumps(value), connection=connection)

    def south_field_triple(self):
        u""" Возвращает подходящее описание поля для South. """
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.TextField"
        args, kwargs = introspector(self)
        # That's our definition!
        return field_class, args, kwargs


